==> v0.37.zip: frostfs-testcases/pytest_tests/testsuites/services/s3_gate/test_s3_tagging.py <==
from random import choice
from string import ascii_letters
from typing import Tuple

import allure
import pytest
from frostfs_testlib.s3 import AwsCliClient, Boto3ClientWrapper, S3ClientWrapper
from frostfs_testlib.steps.s3 import s3_helper
from frostfs_testlib.storage.dataclasses.object_size import ObjectSize
from frostfs_testlib.utils.file_utils import generate_file


def pytest_generate_tests(metafunc: pytest.Metafunc):
    if "s3_client" in metafunc.fixturenames:
        metafunc.parametrize("s3_client", [AwsCliClient, Boto3ClientWrapper], indirect=True)


@pytest.mark.sanity
@pytest.mark.s3_gate
@pytest.mark.s3_gate_tagging
class TestS3GateTagging:
    @staticmethod
    def create_tags(count: int) -> Tuple[list, list]:
        tags = []
        for _ in range(count):
            tag_key = "".join(choice(ascii_letters) for _ in range(8))
            tag_value = "".join(choice(ascii_letters) for _ in range(12))
            tags.append((tag_key, tag_value))
        return tags

    @allure.title("Object tagging (s3_client={s3_client})")
    def test_s3_object_tagging(
        self, s3_client: S3ClientWrapper, bucket: str, simple_object_size: ObjectSize
    ):
        file_path = generate_file(simple_object_size.value)
        file_name = s3_helper.object_key_from_file_path(file_path)

        with allure.step("Put with 3 tags object into bucket"):
            tag_1 = "Tag1=Value1"
            s3_client.put_object(bucket, file_path, tagging=tag_1)
            got_tags = s3_client.get_object_tagging(bucket, file_name)
            assert got_tags, f"Expected tags, got {got_tags}"
            assert got_tags == [{"Key": "Tag1", "Value": "Value1"}], "Tags must be the same"

        with allure.step("Put 10 new tags for object"):
            tags_2 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_2)
            s3_helper.check_tags_by_object(
                s3_client, bucket, file_name, tags_2, [("Tag1", "Value1")]
            )

        with allure.step("Put 10 extra new tags for object"):
            tags_3 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_3)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, tags_3, tags_2)

        with allure.step("Copy one object with tag"):
            copy_obj_path_1 = s3_client.copy_object(bucket, file_name, tagging_directive="COPY")
            s3_helper.check_tags_by_object(s3_client, bucket, copy_obj_path_1, tags_3, tags_2)

        with allure.step("Put 11 new tags to object and expect an error"):
            tags_4 = self.create_tags(11)
            with pytest.raises(Exception, match=r".*Object tags cannot be greater than 10*"):
                # An error occurred (BadRequest) when calling the PutObjectTagging operation: Object tags cannot be greater than 10
                s3_client.put_object_tagging(bucket, file_name, tags=tags_4)

        with allure.step("Put empty tag"):
            tags_5 = []
            s3_client.put_object_tagging(bucket, file_name, tags=tags_5)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, [])

        with allure.step("Put 10 object tags"):
            tags_6 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_6)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, tags_6)

        with allure.step("Delete tags by delete-object-tagging"):
            s3_client.delete_object_tagging(bucket, file_name)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, [])

    @allure.title("Bucket tagging (s3_client={s3_client})")
    def test_s3_bucket_tagging(self, s3_client: S3ClientWrapper, bucket: str):

        with allure.step("Put 10 bucket tags"):
            tags_1 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_1)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_1)

        with allure.step("Put new 10 bucket tags"):
            tags_2 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_2)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_2, tags_1)

        with allure.step("Put 11 new tags to bucket and expect an error"):
            tags_3 = self.create_tags(11)
            with pytest.raises(Exception, match=r".*Object tags cannot be greater than 10.*"):
                # An error occurred (BadRequest) when calling the PutBucketTagging operation (reached max retries: 0): Object tags cannot be greater than 10
                s3_client.put_bucket_tagging(bucket, tags_3)

        with allure.step("Put empty tag"):
            tags_4 = []
            s3_client.put_bucket_tagging(bucket, tags_4)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_4)

        with allure.step("Put new 10 bucket tags"):
            tags_5 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_5)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_5, tags_2)

        with allure.step("Delete tags by delete-bucket-tagging"):
            s3_client.delete_bucket_tagging(bucket)
            s3_helper.check_tags_by_bucket(s3_client, bucket, [])

==> v0.39.zip: frostfs-testcases/pytest_tests/testsuites/services/s3_gate/test_s3_tagging.py <==
from random import choice
from string import ascii_letters
from typing import Tuple

import allure
import pytest
from frostfs_testlib import reporter
from frostfs_testlib.s3 import S3ClientWrapper
from frostfs_testlib.steps.s3 import s3_helper
from frostfs_testlib.storage.dataclasses.object_size import ObjectSize
from frostfs_testlib.utils.file_utils import generate_file


@pytest.mark.s3_gate
@pytest.mark.s3_gate_tagging
class TestS3GateTagging:
    @staticmethod
    def create_tags(count: int) -> Tuple[list, list]:
        tags = []
        for _ in range(count):
            tag_key = "".join(choice(ascii_letters) for _ in range(8))
            tag_value = "".join(choice(ascii_letters) for _ in range(12))
            tags.append((tag_key, tag_value))
        return tags

    @allure.title("Object tagging (s3_client={s3_client})")
    def test_s3_object_tagging(self, s3_client: S3ClientWrapper, bucket: str, simple_object_size: ObjectSize):
        file_path = generate_file(simple_object_size.value)
        file_name = s3_helper.object_key_from_file_path(file_path)

        with reporter.step("Put with 3 tags object into bucket"):
            tag_1 = "Tag1=Value1"
            s3_client.put_object(bucket, file_path, tagging=tag_1)
            got_tags = s3_client.get_object_tagging(bucket, file_name)
            assert got_tags, f"Expected tags, got {got_tags}"
            assert got_tags == [{"Key": "Tag1", "Value": "Value1"}], "Tags must be the same"

        with reporter.step("Put 10 new tags for object"):
            tags_2 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_2)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, tags_2, [("Tag1", "Value1")])

        with reporter.step("Put 10 extra new tags for object"):
            tags_3 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_3)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, tags_3, tags_2)

        with reporter.step("Copy one object with tag"):
            copy_obj_path_1 = s3_client.copy_object(bucket, file_name, tagging_directive="COPY")
            s3_helper.check_tags_by_object(s3_client, bucket, copy_obj_path_1, tags_3, tags_2)

        with reporter.step("Put 11 new tags to object and expect an error"):
            tags_4 = self.create_tags(11)
            with pytest.raises(Exception, match=r".*Object tags cannot be greater than 10*"):
                # An error occurred (BadRequest) when calling the PutObjectTagging operation: Object tags cannot be greater than 10
                s3_client.put_object_tagging(bucket, file_name, tags=tags_4)

        with reporter.step("Put empty tag"):
            tags_5 = []
            s3_client.put_object_tagging(bucket, file_name, tags=tags_5)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, [])

        with reporter.step("Put 10 object tags"):
            tags_6 = self.create_tags(10)
            s3_client.put_object_tagging(bucket, file_name, tags=tags_6)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, tags_6)

        with reporter.step("Delete tags by delete-object-tagging"):
            s3_client.delete_object_tagging(bucket, file_name)
            s3_helper.check_tags_by_object(s3_client, bucket, file_name, [])

    @allure.title("Bucket tagging (s3_client={s3_client})")
    def test_s3_bucket_tagging(self, s3_client: S3ClientWrapper, bucket: str):

        with reporter.step("Put 10 bucket tags"):
            tags_1 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_1)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_1)

        with reporter.step("Put new 10 bucket tags"):
            tags_2 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_2)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_2, tags_1)

        with reporter.step("Put 11 new tags to bucket and expect an error"):
            tags_3 = self.create_tags(11)
            with pytest.raises(Exception, match=r".*Object tags cannot be greater than 10.*"):
                # An error occurred (BadRequest) when calling the PutBucketTagging operation (reached max retries: 0): Object tags cannot be greater than 10
                s3_client.put_bucket_tagging(bucket, tags_3)

        with reporter.step("Put empty tag"):
            tags_4 = []
            s3_client.put_bucket_tagging(bucket, tags_4)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_4)

        with reporter.step("Put new 10 bucket tags"):
            tags_5 = self.create_tags(10)
            s3_client.put_bucket_tagging(bucket, tags_5)
            s3_helper.check_tags_by_bucket(s3_client, bucket, tags_5, tags_2)

        with reporter.step("Delete tags by delete-bucket-tagging"):
            s3_client.delete_bucket_tagging(bucket)
            s3_helper.check_tags_by_bucket(s3_client, bucket, [])
